# Roo Code 3.16.0 Release Notes

*Release notes for Roo Code v3.16.0, published on 2025-05-06.*

This release introduces vertical tab navigation for settings, new API providers ([Groq](/providers/groq) and [Chutes AI](/providers/chutes)), clickable code references, and numerous UI/UX enhancements, alongside various bug fixes and miscellaneous improvements.

## Gemini Model and Caching Updates
- The `gemini-2.5-pro-preview-05-06` model is now available for [Vertex](/providers/vertex) and [Google Gemini](/providers/gemini) providers. Users of the older `gemini-2.5-pro-preview-03-25` will automatically benefit from this newer model, as the previous ID now aliases to the latest version on Google's backend. No configuration changes are needed. (thanks @zetaloop!)
- Prompt caching is now enabled by default for supported Gemini models on the [Vertex](/providers/vertex) and [Google Gemini](/providers/gemini) providers, leading to:
    - **Faster Responses for Repeated Queries**: Gemini remembers previous similar prompts.
    - **Reduced API Usage**: Minimizes redundant API calls.
    - **Simplified Experience with Opt-Out Control**: Active out-of-the-box, but can be disabled in settings.

## Total Settings Navigation Overhaul (thanks @dlab-anton!)

The settings interface has been revamped with a new vertical tab layout for a more efficient and intuitive experience:
    - **One-Click Access:** Navigate between settings sections with a single click via the new vertical tabs in the settings view.
    - **Improved Layout and Clarity:** Settings are now organized in a clear vertical list for better visibility.
        
<img src="/img/v3.16/v3.16.gif" alt="Settings vertical tab navigation" width="800" />

## MCP Service Improvements

    - MCP server errors are now captured and shown in a new "Errors" tab (thanks @robertheadley!)
    - Error logging will no longer break MCP functionality if the server is properly connected (thanks @ksze!)

## Clickable Code References in Chat (thanks @KJ7LNW!)

Navigating code discussed in AI responses is now significantly easier:
    - **Clickable Code and Filenames**: Any `code` or `filename.extension()` mentioned by the AI is now a clickable link.
    - **Jump to Specific Lines**: Links open the relevant file in your editor and navigate directly to the referenced line number.
    - **Streamlined Code Exploration**: Quickly move from AI explanations to the exact spot in your codebase.

<img src="/img/v3.16/v3.16.png" alt="Clickable code references in chat" width="450" />

## Continued UI/UX Improvements (thanks @elianiva!)

General UI improvements for a more consistent, visually appealing, and intuitive experience:
    - **Visually Unified Design**: A more consistent look and feel across settings, prompt interactions, and mode selections.
    - **Improved Theme Adaptability**: Better consistency across different VS Code themes.
    - **Streamlined Interactions**: Tidied up UI elements like mode selection and prompt enhancement areas.
    - **Modernized Icons and Tooltips for Code Blocks**: Code block controls (copy, wrap, expand/collapse) now use crisp Lucide icons and feature translated tooltips for better accessibility.

<img src="/img/v3.16/v3.16-1.png" alt="General UI/UX improvements example" width="800" />

    *These are just a few examples of the many UI/UX improvements in this release.*

## New Provider: Groq Integration (thanks @shariqriazz!)
You can now connect to [Groq](/providers/groq) and utilize their high-speed language models directly within the extension.

## New Provider: Chutes AI Integration (thanks @shariqriazz!)
Support for [Chutes AI](/providers/chutes) has also been added, allowing you to leverage their specialized AI capabilities.

## Misc. Bug Fixes & QOL Improvements

    - Fix migration and persistence of per-mode API profiles (thanks @alasano!)
    - Fix usage of [`path.basename()`](#) in the extension webview (thanks @samhvw8!)
    - Fix display issue of the programming language dropdown in the code block component (thanks @zhangtony239!)
    - Requesty provider fixes (thanks @dtrugman!)
    - Improve accessibility of auto-approve toggles (thanks @Deon588!)
    - You can now toggle the [`terminal.integrated.inheritEnv`](#) VSCode setting directly for the Roo Code settings (thanks @KJ7LNW!)
    - Ensure evals exercises are up-to-date before running evals (thanks @shariqriazz!)
    - Organize provider settings into separate components
    - Add support for tests that use ESM libraries
    - Move environment detail generation to a separate module